% std_readspec() - returns the stored mean power spectrum for an ICA component 
%                  in a specified dataset.  The spectrum is assumed to have been 
%                  saved in a Matlab file, "[dataset_name].icaspec", in the same
%                  directory as the dataset file. If this file doesn't exist,
%                  use std_spec() to create it or a pre-clustering function
%                  (pop_preclust() or std_preclust()) that calls it. 
% Usage:    
%  >> [spec, freqs] = std_readspec(ALLEEG, setindx, component, freqrange, rmsubjmean);  
%
% Inputs:
%   ALLEEG     - a vector of dataset EEG structures (may also be one dataset). 
%                Must contain the dataset of interest (the 'setindx' below).
%   setindx    - [integer] an index of an EEG dataset in the ALLEEG
%                structure for which to read a component spectrum.
%   component  - [integer] index of the component in the selected EEG dataset 
%                for which to return the spectrum
%   freqrange  - [min max in Hz] frequency range to return
%   rmsubjmean - [0|1] remove subject mean spectrum (0 is no and is the default)
%
% Outputs:
%   spec      - the log-power spectrum of the requested ICA component in the
%               specified dataset (in dB)
%   freqs     - vector of spectral frequencies (in Hz)
%
%  See also  std_spec(), pop_preclust(), std_preclust()
%
% Authors:  Arnaud Delorme, Hilit Serby, SCCN, INC, UCSD, February, 2005

%123456789012345678901234567890123456789012345678901234567890123456789012

% Copyright (C) Hilit Serby, SCCN, INC, UCSD, October 11, 2004, hilit@sccn.ucsd.edu
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% $Log: std_readspec.m,v $
% Revision 1.18  2007/02/05 16:17:13  arno
% fix crash for old study subtracting average spectrum
%
% Revision 1.17  2006/11/23 01:12:50  arno
% implement mean spectrum subtraction
%
% Revision 1.16  2006/11/10 00:08:43  arno
% reprogram for channel
%
% Revision 1.15  2006/10/04 23:39:49  toby
% Bug fix courtesy Bas de Kruif
%
% Revision 1.14  2006/03/28 15:38:13  scott
% help msg
%
% Revision 1.13  2006/03/14 02:32:32  scott
% help msg
%
% Revision 1.12  2006/03/11 07:30:01  arno
% freqrange input
%
% Revision 1.11  2006/03/11 07:25:37  arno
% header
%
% Revision 1.10  2006/03/10 16:33:37  arno
% selecting frequency range for reading
%
% Revision 1.9  2006/03/10 00:37:45  arno
% error msg
%
% Revision 1.8  2006/03/09 18:10:38  arno
% *** empty log message ***
%
% Revision 1.7  2006/03/09 18:10:18  arno
% do not use etc field any more
%
% Revision 1.6  2006/03/09 00:42:09  arno
% fix reading file
%
% Revision 1.5  2006/03/09 00:37:31  arno
% now writing matlab fileend
%
% Revision 1.4  2006/03/09 00:03:57  arno
% read spectrum form matlab file
%
% Revision 1.3  2006/03/08 21:06:37  arno
% rename func
%
% Revision 1.2  2006/03/07 22:21:12  arno
% use fullfile
%

function [X, f] = std_readspec(ALLEEG, abset, comp, freqrange, rmsubjmean);

if nargin < 4
    freqrange = [];
end;
if nargin < 5
    rmsubjmean = 0;
end;
    
X = [];

if iscell(comp)
    % find channel indices list
    % -------------------------
    chanind  = [];
    chanlabs = lower({ ALLEEG(abset).chanlocs.labels });
    for index = 1:length(comp)
        tmp = strmatch(lower(comp{index}), chanlabs, 'exact');
        if isempty(tmp)
            error([ 'Channel ''' comp{index} ''' not found in dataset ' int2str(abset)]);
        else    
            chanind = [ chanind tmp ];
        end;
    end;
    filename = fullfile( ALLEEG(abset).filepath,[ ALLEEG(abset).filename(1:end-3) 'datspec']);
    prefix = 'chan';
    inds   = chanind;
elseif comp(1) < 0
    filename = fullfile( ALLEEG(abset).filepath,[ ALLEEG(abset).filename(1:end-3) 'datspec']);
    prefix = 'chan';
    inds   = -comp;
else
    filename = fullfile( ALLEEG(abset).filepath,[ ALLEEG(abset).filename(1:end-3) 'icaspec']);
    prefix = 'comp';
    inds   = comp;
end;

for k=1:length(inds)
    try,
        warning backtrace off;
        if rmsubjmean == 0
             erpstruct = load( '-mat', filename, [ prefix int2str(inds(k)) ], 'freqs' );
        else erpstruct = load( '-mat', filename, [ prefix int2str(inds(k)) ], 'freqs', 'average_spec' );
        end;
        warning backtrace on;
    catch
        error( [ 'Cannot read file ''' filename '''' ]);
    end;

    tmpdat    = getfield(erpstruct, [ prefix int2str(inds(k)) ]);
    if rmsubjmean, 
        if isfield(erpstruct, 'average_spec')
            tmpdat = tmpdat - erpstruct.average_spec; 
        end;
    end;
    if k == 1
        X = zeros(length(comp), length(tmpdat));
    end;
    X(k,:)  = tmpdat;
    f       = getfield(erpstruct, 'freqs');
end;

% select frequency range of interest
% ----------------------------------

%check if freqrange is specified
if ~isempty(freqrange)
    maxind = max(find(f <= freqrange(end)));
    minind = min(find(f >= freqrange(1)));
else
    %if not, use whole spectrum
    maxind = length(f);
    minind = 1;
end

f = f(minind:maxind);
X = X(:,minind:maxind);
%if f(end) < freqrange(end)
%    disp(['Warning! Requested high frequency limit, ' ...
%          num2str(freqrange(end)) 'Hz, is out of bounds.']);
%end
%if f(1) > freqrange(1)
%    disp(['Warning! Requested low frequency limit, ' ...
%          num2str(freqrange(1)) 'Hz, is out of bounds.']);
%end

return;
